import { Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { HttpClient } from '@angular/common/http';

interface ClientInfo {
  property: string;
  value: any;
}

@Component({
  selector: 'app-client-main-view',
  templateUrl: './client-main-view.component.html',
  styleUrl: './client-main-view.component.css'
})
export class ClientMainViewComponent implements OnInit {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  @ViewChild('assistantContainer') assistantContainer!: ElementRef;
  clientUuid: string;
  clientData: any = {};
  isPartitionAssistantVisible: boolean = false;
  isBootImageVisible: boolean = false;
  isDiskUsageVisible: boolean = true;
  generalData: ClientInfo[] = [];
  networkData: ClientInfo[] = [];
  classroomData: ClientInfo[] = [];
  diskUsageData: any[] = [];
  isDiskUsageEmpty: boolean = true;
  loading: boolean = true;  // Variable para controlar el estado de carga

  constructor(private http: HttpClient) {
    const url = window.location.href;
    const segments = url.split('/');
    this.clientUuid = segments[segments.length - 1];
  }

  ngOnInit() {
    this.clientData = history.state.clientData;
    this.updateGeneralData();
    this.updateNetworkData();
    this.updateClassroomData();
    this.calculateDiskUsage();
    this.loading = false;  // Desactivar el estado de carga cuando los datos se hayan procesado
  }

  updateGeneralData() {
    this.generalData = [
      { property: 'Nombre', value: this.clientData?.name || '' },
      { property: 'Uuid', value: this.clientData?.uuid || '' },
      { property: 'IP', value: this.clientData?.ip || '' },
      { property: 'MAC', value: this.clientData?.mac || '' },
      { property: 'Nº de serie', value: this.clientData?.serialNumber || '' },
      { property: 'Netiface', value: this.clientData?.netiface || '' },
      { property: 'Perfil hardware', value: this.clientData?.hardwareProfile?.description || '' },
      { property: 'Menú', value: this.clientData?.menu?.description || '' },
      { property: 'Fecha de creación', value: this.clientData?.createdAt || '' },
      { property: 'Creado por', value: this.clientData?.createdBy || '' }
    ];
  }

  updateNetworkData() {
    this.networkData = [
      { property: 'Perfil hardware', value: this.clientData?.hardwareProfile?.description || '' },
      { property: 'Subred', value: this.clientData?.subnet || '' },
      { property: 'OGlive', value: '' },
      { property: 'Autoexec', value: '' },
      { property: 'Repositorio', value: '' },
      { property: 'Validación', value: this.clientData?.organizationalUnit?.networkSettings?.validation || '' },
      { property: 'Fecha de creación', value: this.clientData?.createdAt || '' },
      { property: 'Creado por', value: this.clientData?.createdBy || '' }
    ];
  }

  updateClassroomData() {
    this.classroomData = [
      { property: 'Url servidor proxy', value: this.clientData?.organizationalUnit?.networkSettings?.proxy || '' },
      { property: 'IP DNS', value: this.clientData?.organizationalUnit?.networkSettings?.dns || '' },
      { property: 'Máscara de red', value: this.clientData?.organizationalUnit?.networkSettings?.netmask || '' },
      { property: 'Router', value: this.clientData?.organizationalUnit?.networkSettings?.router || '' },
      { property: 'NTP', value: this.clientData?.organizationalUnit?.networkSettings?.ntp || '' },
      { property: 'Modo p2p', value: this.clientData?.organizationalUnit?.networkSettings?.p2pMode || '' },
      { property: 'Tiempo p2p', value: this.clientData?.organizationalUnit?.networkSettings?.p2pTime || '' },
      { property: 'IP multicast', value: this.clientData?.organizationalUnit?.networkSettings?.mcastIp || '' },
      { property: 'Modo multicast', value: this.clientData?.organizationalUnit?.networkSettings?.mcastMode || '' },
      { property: 'Puerto multicast', value: this.clientData?.organizationalUnit?.networkSettings?.mcastPort || '' },
      { property: 'Velocidad multicast', value: this.clientData?.organizationalUnit?.networkSettings?.mcastSpeed || '' },
      { property: 'Perfil hardware', value: this.clientData?.organizationalUnit?.networkSettings?.hardwareProfile?.description || '' }
    ];
  }

  calculateDiskUsage() {
    const diskUsageMap = new Map<number, { total: number, used: number }>();

    this.clientData.partitions.forEach((partition: any) => {
      const diskNumber = partition.diskNumber;
      if (!diskUsageMap.has(diskNumber)) {
        diskUsageMap.set(diskNumber, { total: 0, used: 0 });
      }

      const diskData = diskUsageMap.get(diskNumber);
      if (partition.partitionNumber === 0) {
        diskData!.total = partition.size / (1024 * 1024);
      } else {
        diskData!.used += partition.size / (1024 * 1024);
      }
    });

    this.diskUsageData = Array.from(diskUsageMap.entries()).map(([diskNumber, { total, used }]) => {
      const percentage = total > 0 ? Math.round((used / total) * 100) : 0;
      return { diskNumber, total, used, percentage };
    });
    this.isDiskUsageEmpty = this.diskUsageData.length === 0;
  }

  togglePartitionAssistant() {
    this.isPartitionAssistantVisible = !this.isPartitionAssistantVisible;
    this.isBootImageVisible = false;
    if (this.isPartitionAssistantVisible) {
      setTimeout(() => {
        this.assistantContainer.nativeElement.scrollIntoView({ behavior: 'smooth' });
      }, 200);
    }
  }

  showBootImage() {
    this.isBootImageVisible = !this.isBootImageVisible;
    this.isPartitionAssistantVisible = false;
    if (this.isBootImageVisible) {
      setTimeout(() => {
        this.assistantContainer.nativeElement.scrollIntoView({ behavior: 'smooth' });
      }, 0);
    }
  }
}
